package org.mime4j;

import java.io.InputStream;
import java.io.IOException;

/**
 * InputStream that shields its underlying input stream from
 * being closed.
 */
public class CloseShieldInputStream extends InputStream {

    /**
     * Underlying InputStream
     */
    private InputStream is;

    public CloseShieldInputStream(InputStream is) {
        this.is = is;
    }

    public InputStream getUnderlyingStream() {
        return is;
    }

    public int read() throws IOException {
        checkIfClosed();
        return is.read();
    }

    public int available() throws IOException {
        checkIfClosed();
        return is.available();
    }

    public void close() throws IOException {
        is = null;
    }

    public synchronized void reset() throws IOException {
        checkIfClosed();
        is.reset();
    }

    public boolean markSupported() {
        if (is == null)
            return false;
        return is.markSupported();
    }

    public synchronized void mark(int readlimit) {
        if (is != null)
            is.mark(readlimit);
    }

    public long skip(long n) throws IOException {
        checkIfClosed();
        return is.skip(n);
    }

    public int read(byte b[]) throws IOException {
        checkIfClosed();
        return is.read(b);
    }

    public int read(byte b[], int off, int len) throws IOException {
        checkIfClosed();
        return is.read(b, off, len);
    }

    private void checkIfClosed() throws IOException {
        if (is == null)
            throw new IOException("Stream is closed");
    }
}
