/*
 *  Copyright 2002-2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.mime4j;

import java.util.BitSet;

public class HeaderLexer {

    public static final int OPTION_RETURN_WHITESPACE = 0;
    public static final int OPTION_SUPPRESS_NON_ASCII = 1;

    private final BitSet options;

    private final byte[] data;
    private int pos;

    public HeaderLexer(byte[] headerValue, BitSet options) {
        this.data = headerValue;
        this.options = options;
    }

    public HeaderToken nextToken() {

        while (true) {

            byte b = (byte)la(0);

            switch (b) {

                case -1:
                    return new HeaderToken(HeaderToken.EOF, null);

                case '(':
                case ' ':
                case '\t':
                    consumeCommentAndWhitespace();
                    if (options.get(OPTION_RETURN_WHITESPACE))
                        return new HeaderToken(HeaderToken.WS, " ");
                    else
                        break;

                case '"':
                    return matchQuote();

                default:
                    return matchAtom();
            }
        }
    }

    /**
     * Lookahead the specified number of bytes.
     */
    private int la(int count) {
        if (pos + count >= data.length)
            return -1;
        else
            return data[pos + count] & 0xFF;
    }

    private int nextByte() {
        if (pos < data.length)
            return data[pos++];
        else
            return -1;
    }

    private void consumeCommentAndWhitespace() {

        final int NOT_IN_COMMENT = 0;
        final int IN_COMMENT = 1;

        while (pos < data.length) {

            byte b = data[pos++];

            // TODO: impl

        }
    }

    private HeaderToken matchQuote() {

        StringBuffer out = new StringBuffer();

        pos++;

        final int NOT_IN_QP = 0;
        final int IN_QP = 1;

        int state = NOT_IN_QP;

        loop:
        while (pos < data.length) {

            byte b = data[pos++];

            if (state == NOT_IN_QP) {
                switch (b) {
                    case '\\':
                        state = IN_QP;
                        continue;

                    case '"':
                        break loop;

                    default:
                        out.append((char)b);
                        break;
                }
            }
            else if (state == IN_QP) {
                out.append((char)b);
                state = NOT_IN_QP;
            }
        }

        return new HeaderToken(HeaderToken.QUOTED_STRING, out.toString());

    }

    private HeaderToken matchAtom() {
        // TODO: impl
        return null;
    }

}
