/*
 *  Copyright 2002-2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.mime4j;

import java.io.InputStream;
import java.io.IOException;

public interface MIMEEventCallbacks {

    /**
     * Called when the enclosing message is not MIME.
     */
    public void onNonMimeBody(Headers headers, InputStream stream) throws IOException;

    /**
     * Called any time any headers are encountered, whether on a
     * message, submessage, or body part.
     */
    public void onHeaders(Headers headers) throws IOException;

    /**
     * Called any time a MIME preamble is encountered.
     * MIME compliant readers should ignore this area.
     */
    public void onPreamble(InputStream stream) throws IOException;

    /**
     * Called any time a MIME epilogue is encountered.
     * MIME compliant readers should ignore this area.
     */
    public void onEpilogue(InputStream stream) throws IOException;

    /**
     * Called any time the start of a multipart/* is detected.
     */
    public void onMultipartStart(Headers headers) throws IOException;

    /**
     * Called when a multipart/* part ends.
     */
    public void onMultipartEnd() throws IOException;

    /**
     * Called any time the start of an embedded message is detected.
     */
    public void onMessageStart(Headers headers) throws IOException;

    /**
     * Called any time an embedded message ends.
     */
    public void onMessageEnd() throws IOException;

    /**
     * Called when a non-multipart body part is detected.  The stream
     * (non-writable, non-seekable) represents the data of the body
     * part.  If the body part was encoded using "quoted-printable"
     * or "base64", it is decoded automatically by the stream.
     *
     * Note that if the body part is text/*, care should be taken to
     * apply the correct charset to the stream.
     */
    public void onDiscreteBodyPart(Headers headers, InputStream stream) throws IOException;

}
